"""Public methods for library users."""

import tempfile
from pathlib import Path

from .lib import edit, write_file

DEFAULT_TEMPFILE = "tempfile"


def edit_file(filepath):
    """
    Open an existing file in the system's default editor for the user to edit.
    The saved contents of the file will be returned when the editor is closed.

    :param filepath: Can be absolute or relative (including just a file name).
    :return: Contents of the file when the editor is closed.
    """

    return edit(filepath)


def edit_temp(contents="", name=""):
    """
    Create a temporary file and open it in the system's default editor for the
    user to edit. The saved contents of the file will be returned when the
    editor is closed.

    :param contents: Pre-fill the file with the given text.
    :param name: Ensure that the temp filename has the given name.
    :return: Contents of the file when the editor is closed.
    """

    # Create a temp file, ensure it has requested name and contents
    td = tempfile.TemporaryDirectory()
    tfpath = Path(td.name) / (name or DEFAULT_TEMPFILE)
    write_file(tfpath, contents)

    # Edit interactively
    return edit(tfpath)



"""Internal functions."""

import os
import subprocess
import shlex


class EditorError(RuntimeError):
    """An error with editing a file with the editor."""

    pass


def edit(filepath):
    """Open the given file in an editor for the user to edit.

    Assumptions:
    1. The editor command is not too complex for shlex to handle.
    2. The path of the file to be edited is the final argument.

    :param filepath: Path to the file.
    :return: Contents of the file."""

    parts = shlex.split(editor(), posix=True) + [filepath]
    subprocess.call(parts, close_fds=True)

    with open(filepath) as f:
        return f.read()


def editor() -> str:
    """Get system's editor."""

    # VISUAL is preferable to EDITOR on virtually all modern systems
    # See https://unix.stackexchange.com/q/4859
    editor_cmd = os.environ.get("VISUAL") or os.environ.get("EDITOR")

    if editor_cmd:
        return editor_cmd
    else:
        raise EditorError(
            "Both $VISUAL and $EDITOR are unset, could not pick "
            "an appropriate editor."
        )


def write_file(path, contents=""):
    """Create a file. If contents are given, they will be written to it."""
    with open(path, 'w') as f:
        if contents:
            f.write(contents)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['openeditor']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'openeditor',
    'version': '0.3.1',
    'description': 'Edit files with your $EDITOR, like git commit does.',
    'long_description': '# `openeditor`\nEdit files with your `$EDITOR`, like git commit does.\n\n## Usage\nInstall with: `pip install openeditor`\n\n```\n# Let user edit file\ns = openeditor.edit_file("path/to/my/file.txt")\nprint("The file now contains:\\n" + s)\n\n# Use a temp file\ns = openeditor.edit_file(\n    "# Please edit this file, save and close editor when done", \n    "path/to/my/file.txt"\n)\nprint("The file now contains:\\n" + s) \n```\n\nThe editor is obtained from, in order of precedence:\n\n* `$VISUAL`\n* `$EDITOR`\n\nIf neither of these provide a useful editor, an exception will be thrown.\n\n## Limitations\n`openeditor` expects an editor string similar to `EDITOR=cmd` such that:\n\n1. `cmd file.txt` (filename as the final argument) is a correct way of editing `file.txt`.\n2. `cmd` is not too complex. Simple things like space-separated flags (eg. `EDITOR="vim -n"`) are fine but advanced shell magic may break.\n\nIf your `cmd` **does** need to be more complex, one possible workaround is to write a wrapper script that presents a compatible command-line interface to `openeditor` and invokes the full command as appopriate.',
    'author': 'Azat Akhmetov',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/metov/openeditor',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
